//////////////////////////////////////////////////////////////////////////////
// nitrogenconfigurationui.cpp
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenconfigurationui.h"
#include "nitrogenconfigurationui.moc"
#include "nitrogenexceptionlistwidget.h"
#include "../nitrogenconfiguration.h"

#include <kdeversion.h>

#include <QtGui/QLabel>
#include <QtGui/QLayout>
#include <QtGui/QGroupBox>

#include <KLocale>
#include <KTabWidget>

namespace Nitrogen
{

  //_________________________________________________________
  NitrogenConfigurationUI::NitrogenConfigurationUI( QWidget* parent ):
    QWidget( parent ),
    titleAlignment(0),
    buttonSize(0),
    buttonType(0),
    frameBorder(0),
    blendColor(0),
    separatorType(0),
    sizeGripMode(0),
    showStripes(0),
    overwriteColors(0),
    titleOutline(0),
    useOxygenShadows(0),
    useAnimations(0),
    animationsDuration( 0 ),
    aboutNitrogen(0)
  { setupUI(); }

  //_________________________________________________________
  void NitrogenConfigurationUI::setupUI( void )
  {

    QVBoxLayout* mainLayout = new QVBoxLayout( this );
    mainLayout->setMargin(0);

    // tab widget for basic and advanced mode
    KTabWidget* tab( new KTabWidget( this ) );
    mainLayout->addWidget( tab );

    // basic configuration
    {
      QWidget *basicWidget = new QWidget();
      int index = tab->addTab( basicWidget, i18n("&General") );
      tab->setTabToolTip( index, i18n( "Basic window decoration configuration options" ) );

      QVBoxLayout* vboxLayout = new QVBoxLayout();
      basicWidget->setLayout( vboxLayout );

      QGridLayout* gridLayout = new QGridLayout();
      gridLayout->setMargin(0);
      vboxLayout->addLayout( gridLayout );

      gridLayout->setColumnStretch(2, 1);

      // frame border
      QLabel* label;
      gridLayout->addWidget( label = new QLabel( i18n("Border size:"), basicWidget ), 0, 0, 1, 1);
      gridLayout->addWidget( frameBorder = new QComboBox(basicWidget), 0, 1, 1, 1);
      frameBorder->setObjectName(QString::fromUtf8("frameBorder"));
      frameBorder->insertItems(0, QStringList()
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderNone, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderNoSide, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderTiny, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderDefault, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderLarge, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderVeryLarge, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderHuge, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderVeryHuge, true )
        << NitrogenConfiguration::frameBorderName( NitrogenConfiguration::BorderOversized, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( frameBorder );

      // title alignment
      gridLayout->addWidget( label = new QLabel( i18n("Title alignment:"), basicWidget ), 1, 0, 1, 1 );
      gridLayout->addWidget( titleAlignment = new QComboBox(basicWidget), 1, 1, 1, 1 );
      titleAlignment->setObjectName(QString::fromUtf8("titleAlignment"));
      titleAlignment->insertItems(0, QStringList()
        << NitrogenConfiguration::titleAlignmentName( Qt::AlignLeft, true )
        << NitrogenConfiguration::titleAlignmentName( Qt::AlignHCenter, true )
        << NitrogenConfiguration::titleAlignmentName( Qt::AlignRight, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( titleAlignment );

      // button size
      gridLayout->addWidget( label = new QLabel( i18n("Button size:"), basicWidget ), 2, 0, 1, 1 );
      gridLayout->addWidget( buttonSize = new QComboBox(basicWidget), 2, 1, 1, 1 );
      buttonSize->setObjectName(QString::fromUtf8("buttonSize"));
      buttonSize->insertItems(0, QStringList()
        << NitrogenConfiguration::buttonSizeName( NitrogenConfiguration::ButtonSmall, true )
        << NitrogenConfiguration::buttonSizeName( NitrogenConfiguration::ButtonDefault, true )
        << NitrogenConfiguration::buttonSizeName( NitrogenConfiguration::ButtonLarge, true )
        << NitrogenConfiguration::buttonSizeName( NitrogenConfiguration::ButtonHuge, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( buttonSize );

      // animations
      {

        vboxLayout->addWidget( useAnimations = new QCheckBox( i18n("Animate transitions between active and inactive window state"), basicWidget) );
        useAnimations->setObjectName(QString::fromUtf8("useAnimations"));
        useAnimations->setWhatsThis(i18n(
          "When enabled, transitions between active and inactive window state are progressive rather than instantaneous"));

        QHBoxLayout* hLayout( new QHBoxLayout() );
        hLayout->setMargin(0);
        vboxLayout->addLayout( hLayout );

        hLayout->addSpacing( 20 );
        hLayout->addWidget( label = new QLabel( i18n( " Duration:" ), basicWidget ) );
        hLayout->addWidget( animationsDuration = new QSpinBox( basicWidget ) );
        animationsDuration->setMinimum( 50 );
        animationsDuration->setMaximum( 500 );
        animationsDuration->setEnabled( false );
        hLayout->addStretch( 1 );

        label->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
        label->setBuddy( animationsDuration );

        connect( useAnimations, SIGNAL( toggled( bool ) ), animationsDuration, SLOT( setEnabled( bool ) ) );

      }

      // active window title outline
      vboxLayout->addWidget( titleOutline = new QCheckBox( i18n("Outline active window title"), basicWidget) );
      titleOutline->setObjectName(QString::fromUtf8("titleOutline"));
      titleOutline->setWhatsThis(i18n(
        "When enabled, an additional frame is shown around the active window as well as its title"));

      // show stripes
      vboxLayout->addWidget( showStripes = new QCheckBox( i18n("Show stripes next to the active window title"), basicWidget) );
      showStripes->setObjectName(QString::fromUtf8("showStripes"));
      showStripes->setWhatsThis(i18n(
        "When enabled, this option increases the visibility of the window titlebar by showing stripes"));

    }


    // advanced configuration
    {
      QWidget *advancedWidget = new QWidget();
      int index = tab->addTab( advancedWidget, i18n( "&Advanced" ) );
      tab->setTabToolTip( index, i18n( "Advanced window decoration configuration options" ) );

      QVBoxLayout* vboxLayout = new QVBoxLayout();
      advancedWidget->setLayout( vboxLayout );

      QGridLayout* gridLayout = new QGridLayout();
      gridLayout->setMargin(0);
      vboxLayout->addLayout( gridLayout );

      gridLayout->setColumnStretch(2, 1);

      // button type
      QLabel* label;
      gridLayout->addWidget( label = new QLabel( i18n("Button style:"), advancedWidget ), 0, 0, 1, 1 );
      gridLayout->addWidget( buttonType = new QComboBox(advancedWidget), 0, 1, 1, 1 );
      buttonType->setObjectName(QString::fromUtf8("buttonType"));
      buttonType->insertItems(0, QStringList()
        << NitrogenConfiguration::buttonTypeName( NitrogenConfiguration::ButtonKde42, true )
        << NitrogenConfiguration::buttonTypeName( NitrogenConfiguration::ButtonKde43, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( buttonType );

      // title bar blending
      gridLayout->addWidget( label = new QLabel( i18n("Background style:" ), advancedWidget ), 1, 0, 1, 1 );
      gridLayout->addWidget( blendColor = new QComboBox(advancedWidget), 1, 1, 1, 1 );
      blendColor->setObjectName(QString::fromUtf8("blendColor"));
      blendColor->insertItems(0, QStringList()
        << NitrogenConfiguration::blendColorName( NitrogenConfiguration::NoBlending, true )
        << NitrogenConfiguration::blendColorName( NitrogenConfiguration::RadialBlending, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( blendColor );

      // separator
      gridLayout->addWidget( label = new QLabel( i18n("Separator display:" ), advancedWidget ), 2, 0, 1, 1 );
      gridLayout->addWidget( separatorType = new QComboBox(advancedWidget), 2, 1, 1, 1 );
      separatorType->setObjectName(QString::fromUtf8("separatorType"));
      separatorType->insertItems(0, QStringList()
        << NitrogenConfiguration::separatorTypeName( NitrogenConfiguration::SeparatorAlways, true )
        << NitrogenConfiguration::separatorTypeName( NitrogenConfiguration::SeparatorActive, true )
        << NitrogenConfiguration::separatorTypeName( NitrogenConfiguration::SeparatorNever, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( separatorType );

      // draw size grip
      gridLayout->addWidget( label = new QLabel( i18n("Extra Size grip display:"), advancedWidget ), 3, 0, 1, 1 );
      gridLayout->addWidget( sizeGripMode = new QComboBox(advancedWidget), 3, 1, 1, 1 );
      sizeGripMode->setObjectName(QString::fromUtf8("sizeGripMode"));
      sizeGripMode->setWhatsThis(i18n(
        "The extra size grip is a small triangle shown in the bottom-right corner of a window \n"
        "which allows to resize the window. This option controls in which case this size grip \n"
        "is shown."));
      sizeGripMode->insertItems(0, QStringList()
        << NitrogenConfiguration::sizeGripModeName( NitrogenConfiguration::SizeGripNever, true )
        << NitrogenConfiguration::sizeGripModeName( NitrogenConfiguration::SizeGripWhenNeeded, true )
        << NitrogenConfiguration::sizeGripModeName( NitrogenConfiguration::SizeGripAlways, true )
        );

      label->setAlignment( Qt::AlignRight|Qt::AlignVCenter );
      label->setBuddy( sizeGripMode );

      // oxygen shadow
      vboxLayout->addWidget( useOxygenShadows = new QCheckBox( i18n("Glow active window" ), advancedWidget ) );
      useOxygenShadows->setObjectName(QString::fromUtf8("useOxygenShadows"));
      useOxygenShadows->setWhatsThis(i18n(
        "When this option is enabled, oxygen signature blue glow is used for the active window shadow."));

      // overwrite colors
      vboxLayout->addWidget( overwriteColors = new QCheckBox( i18n("Blend title bar colors with window contents"), advancedWidget) );
      overwriteColors->setObjectName(QString::fromUtf8("overwriteColors"));
      overwriteColors->setWhatsThis(i18n(
        "When enabled, window colors are used in place of default title bar colors to draw the decoration"));

      vboxLayout->addStretch( 1 );

      connect( titleOutline, SIGNAL(toggled( bool )), overwriteColors, SLOT( setDisabled( bool ) ) );
      connect( titleOutline, SIGNAL(toggled( bool )), showStripes, SLOT( setDisabled( bool ) ) );
      connect( titleOutline, SIGNAL(toggled( bool )), separatorType, SLOT( setDisabled( bool ) ) );



    }

    // exceptions
    {
      exceptions = new NitrogenExceptionListWidget();
      exceptions->setObjectName(QString::fromUtf8("exceptions"));
      int index = tab->addTab( exceptions, i18n( "&Window-Specific" ) );
      tab->setTabToolTip( index, i18n( "Configure window specific decoration options, based on window name" ) );
    }

    QMetaObject::connectSlotsByName(this);

    connect( titleAlignment, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( buttonSize, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( buttonType, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( frameBorder, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( blendColor, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( titleOutline, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( separatorType, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
    connect( sizeGripMode, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );

    connect( showStripes, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( overwriteColors, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( titleOutline, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( useOxygenShadows, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( useAnimations, SIGNAL(clicked()), SIGNAL(changed()) );
    connect( animationsDuration, SIGNAL(valueChanged( int )), SIGNAL(changed()) );
    connect( exceptions, SIGNAL(changed()), SIGNAL(changed()) );


  }

}
